package cn.sh.stc.sict.cloud.upms.controller.web;

import cn.hutool.core.util.StrUtil;
import cn.sh.stc.sict.cloud.upms.model.SysLog;
import cn.sh.stc.sict.cloud.upms.service.SysLogService;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import cn.sh.stc.sict.cloud.common.core.util.R;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.*;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;


/**
 * 系统日志
 *
 * @author F_xh。
 * @date 2020-12-27 11:34:55
 */
@RestController
@RequestMapping("/syslog")
@AllArgsConstructor
@Slf4j
@Api(value = "系统日志 controller", tags = "系统日志")
public class SysLogController {

    private SysLogService sysLogService;

    /**
     * 分页查询
     *
     * @param page   分页对象
     * @param sysLog 系统日志
     * @return
     */
    @ApiOperation("分页查询 系统日志")
    @GetMapping("/page")
    public R getSysLogPage(Page page, SysLog sysLog) {
        LambdaQueryWrapper<SysLog> wrapper = Wrappers.lambdaQuery();
        wrapper.like(StrUtil.isNotBlank(sysLog.getTitle()), SysLog::getTitle, sysLog.getTitle())
                .like(StrUtil.isNotBlank(sysLog.getMethod()), SysLog::getMethod, sysLog.getMethod())
                .like(StrUtil.isNotBlank(sysLog.getParams()), SysLog::getParams, sysLog.getParams())
                .orderByDesc(SysLog::getCreateTime);
        return new R<>(sysLogService.page(page, wrapper));
    }

    /**
     * 通过id查询系统日志
     *
     * @param id id
     * @return R
     */
    @ApiOperation("通过id查询 系统日志")
    @GetMapping("/{id}")
    public R getById(@PathVariable("id") Long id) {
        return new R<>(sysLogService.getById(id));
    }

    /**
     * 新增系统日志
     * @param sysLog 系统日志
     * @return R
     */
    // @ApiOperation("新增系统日志")
    // @PostMapping
    // public R save(@RequestBody SysLog sysLog){
    //    return new R<>(sysLogService.save(sysLog));
    // }

    /**
     * 修改系统日志
     * @param sysLog 系统日志
     * @return R
     */
    // @ApiOperation("修改系统日志")
    // @PutMapping
    // public R updateById(@RequestBody SysLog sysLog){
    //     return new R<>(sysLogService.updateById(sysLog));
    // }

    /**
     * 通过id删除系统日志
     * @param id id
     * @return R
     */
    // @ApiOperation("删除系统日志")
    // @DeleteMapping("/{id}")
    // public R removeById(@PathVariable Long id){
    //    return new R<>(sysLogService.removeById(id));
    // }

}
