package cn.sh.stc.sict.cloud.upms.controller.web;

import cn.hutool.core.util.StrUtil;
import cn.sh.stc.sict.cloud.common.core.util.R;
import cn.sh.stc.sict.cloud.upms.model.SysWxLog;
import cn.sh.stc.sict.cloud.upms.service.SysWxLogService;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;


/**
 * 系统日志
 *
 * @author F_xh。
 * @date 2020-12-27 11:34:55
 */
@RestController
@RequestMapping("/wx/syslog")
@AllArgsConstructor
@Slf4j
@Api(tags = "C端系统日志")
public class SysWxLogController {

    private SysWxLogService sysWxLogService;

    /**
     * 分页查询
     *
     * @param page   分页对象
     * @return
     */
    @ApiOperation("分页查询 系统日志")
    @GetMapping("/page")
    public R getSysLogPage(Page page, SysWxLog sysWxLog) {
        LambdaQueryWrapper<SysWxLog> wrapper = Wrappers.lambdaQuery();
        wrapper.like(StrUtil.isNotBlank(sysWxLog.getTitle()), SysWxLog::getTitle, sysWxLog.getTitle())
                .like(StrUtil.isNotBlank(sysWxLog.getMethod()), SysWxLog::getMethod, sysWxLog.getMethod())
                .like(StrUtil.isNotBlank(sysWxLog.getParams()), SysWxLog::getParams, sysWxLog.getParams())
                .orderByDesc(SysWxLog::getCreateTime);
        return new R<>(sysWxLogService.page(page, wrapper));
    }

    /**
     * 通过id查询系统日志
     *
     * @param id id
     * @return R
     */
    @ApiOperation("通过id查询 系统日志")
    @GetMapping("/{id}")
    public R getById(@PathVariable("id") Long id) {
        return new R<>(sysWxLogService.getById(id));
    }

    /**
     * 新增系统日志
     * @param sysLog 系统日志
     * @return R
     */
    // @ApiOperation("新增系统日志")
    // @PostMapping
    // public R save(@RequestBody SysLog sysLog){
    //    return new R<>(sysLogService.save(sysLog));
    // }

    /**
     * 修改系统日志
     * @param sysLog 系统日志
     * @return R
     */
    // @ApiOperation("修改系统日志")
    // @PutMapping
    // public R updateById(@RequestBody SysLog sysLog){
    //     return new R<>(sysLogService.updateById(sysLog));
    // }

    /**
     * 通过id删除系统日志
     * @param id id
     * @return R
     */
    // @ApiOperation("删除系统日志")
    // @DeleteMapping("/{id}")
    // public R removeById(@PathVariable Long id){
    //    return new R<>(sysLogService.removeById(id));
    // }

}
