package cn.sh.stc.sict.cloud.upms.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.ArrayUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import cn.sh.stc.sict.cloud.common.core.constant.Constant;
import cn.sh.stc.sict.cloud.common.core.constant.RedisCacheConstant;
import cn.sh.stc.sict.cloud.common.core.constant.UserConstant;
import cn.sh.stc.sict.cloud.common.core.constant.enums.LoginTypeEnum;
import cn.sh.stc.sict.cloud.common.core.dto.WDUserInfo;
import cn.sh.stc.sict.cloud.common.core.dto.XSZYUserInfo;
import cn.sh.stc.sict.cloud.common.core.util.NumberUtil;
import cn.sh.stc.sict.cloud.common.core.util.SsbUtil;
import cn.sh.stc.sict.cloud.common.core.util.WoaUtil;
import cn.sh.stc.sict.cloud.upms.dao.SysUserBaseMapper;
import cn.sh.stc.sict.cloud.upms.dto.UserDTO;
import cn.sh.stc.sict.cloud.upms.dto.UserInfo;
import cn.sh.stc.sict.cloud.upms.model.HpRole;
import cn.sh.stc.sict.cloud.upms.model.SysUserBase;
import cn.sh.stc.sict.cloud.upms.service.HpRoleService;
import cn.sh.stc.sict.cloud.upms.service.HpUserRoleService;
import cn.sh.stc.sict.cloud.upms.service.SysUserBaseService;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringPool;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import me.chanjar.weixin.common.bean.oauth2.WxOAuth2AccessToken;
import me.chanjar.weixin.common.error.WxErrorException;
import me.chanjar.weixin.mp.api.WxMpService;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

@Slf4j
@Service("sysUserBaseService")
@AllArgsConstructor
public class SysUserBaseServiceImpl extends ServiceImpl<SysUserBaseMapper, SysUserBase> implements SysUserBaseService {

    private final SysUserBaseMapper sysUserBaseMapper;
    private WxMpService wxMpService;
    private HpUserRoleService hpUserRoleService;
    private HpRoleService hpRoleService;
    private static final PasswordEncoder ENCODER = new BCryptPasswordEncoder();
    private final StringRedisTemplate stringRedisTemplate;

    @Override
    public UserInfo getUserInfo(String appId, String username) {

        LambdaQueryWrapper<SysUserBase> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(SysUserBase::getPhone, username).or().eq(SysUserBase::getUserName, username);
        SysUserBase user = this.getOne(wrapper);
        if (user != null) {
            UserInfo info = new UserInfo();
            info.setSysUserBase(user);
            info.setOpenId(user.getOpenId());
            List<HpRole> roleList = hpRoleService.listByUserId(user.getId());
            if (CollUtil.isNotEmpty(roleList)) {
                // 设置角色列表 （ID）
                List<String> roleIds = roleList.stream().map(HpRole::getRoleCode)
                        .collect(Collectors.toList());
                info.setRoles(ArrayUtil.toArray(roleIds, String.class));
            }

            return info;
        }
        return null;
    }

    @Override
    public UserInfo getUserInfoBySocial(String appId, String inStr) throws WxErrorException {
        log.error("=================> appId = {}, inStr = {}", appId, inStr);
        // code = 011Eg6Ga1UOXeA0R5wHa16dkab2Eg6G7
        String[] split = inStr.split(StringPool.AT);
        LoginTypeEnum loginType = Optional.ofNullable(LoginTypeEnum.match(split[0])).orElse(LoginTypeEnum.WECHAT);
        switch (loginType) {
            // 手机号登录
            case SMS:
                return this.getByPhone(split[2]);
            // 随身办登陆
            case SSB:
                return this.getBySSBToken(split[0], split[2]);
            // 香山中医院公众号登陆
            // inStr = 公众号标识符@公众号appId@token@hospitalCode
            case WOA_NL:
                return this.getByXszyWoa(split);
            // 微信登录
            default:
                WxOAuth2AccessToken token = wxMpService.getOAuth2Service().getAccessToken(split[2]);
                return this.getByOpenId(token.getOpenId());
        }
    }


    /**
     * 获取香山中医医院公众号的用户信息
     *
     * @param infoArray 登陆信息 ["公众号标识符", "公众号appId", "token"]
     * @return
     */
    private UserInfo getByXszyWoa(String[] infoArray) {
        String source = infoArray[0];
        String appId = infoArray[1];
        String token = infoArray[2];
        String hospitalCode = infoArray[3];

        XSZYUserInfo xszyUserInfo = WoaUtil.getXszyUserInfo(token);
        String xszyUserInfoStr = JSONUtil.toJsonStr(xszyUserInfo);
        log.error("woa.login.xszyUser = {}", xszyUserInfoStr);
        if (ObjectUtil.isNull(xszyUserInfo) || StrUtil.isBlank(xszyUserInfo.getUserId())) {
            return null;
        }

        // 根据用户ID和手机号查询当前用户是否已注册过
        LambdaQueryWrapper<SysUserBase> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(SysUserBase::getOpenId, xszyUserInfo.getUserId())
                .or()
                .eq(SysUserBase::getPhone, xszyUserInfo.getMobile())
                .orderByDesc(SysUserBase::getUpdateTime)
                .last("limit 1");
        SysUserBase user = this.getOne(wrapper);

        // 未注册用户默认注册
        if (user == null) {
            user = new SysUserBase();
            user.setUserName(xszyUserInfo.getMobile());
            user.setOpenId(xszyUserInfo.getUserId());
            user.setName(xszyUserInfo.getPatientName());
            // 证件类型为1时，证件号为身份证号
            if (Constant.STRING_YES.equals(xszyUserInfo.getCertificateType())) {
                user.setIdCard(xszyUserInfo.getCertificate());
            }
            //user.setSex(userInfo.getGender());
            user.setHeadimg(Constant.DEFAULT_AVATAR_IMG);
            user.setSource(source);
            user.setStatus(Constant.BYTE_NO);
            user.setPasswd(ENCODER.encode(Constant.DEFAULT_PASSWORD));
            user.setPhone(xszyUserInfo.getMobile());
            user.setAppId(appId);

            this.save(user);
        }

        UserInfo info = new UserInfo();
        info.setSysUserBase(user);
        info.setOpenId(xszyUserInfo.getUserId());

        // 设置医疗机构代码，后续查询只显示当前医疗机构下的查询结果
        info.setHospitalCode(hospitalCode);
        info.setXszyUserInfo(xszyUserInfoStr);
        return info;
    }


    private UserInfo getBySSBToken(String source, String token) {
        WDUserInfo wdUser = SsbUtil.getUserInfo(token);
        log.error("ssb.login.wdUser = {}", JSONUtil.toJsonStr(wdUser));
        if (ObjectUtil.isNull(wdUser) || StrUtil.isBlank(wdUser.getMobile())) {
            return null;
        }
        LambdaQueryWrapper<SysUserBase> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(SysUserBase::getPhone, wdUser.getMobile());
        SysUserBase user = this.getOne(wrapper);
        // 未注册用户默认注册
        if (user == null) {
            user = new SysUserBase();
            user.setUserName(wdUser.getUsername());
            user.setOpenId(wdUser.getMobile());
            user.setName(wdUser.getName());
            user.setIdCard(wdUser.getPersoncard());
            user.setSex(wdUser.getGender());
            user.setHeadimg(wdUser.getAvatar());
            user.setSource(source);
            user.setStatus(Constant.BYTE_NO);
            user.setPasswd(ENCODER.encode(Constant.DEFAULT_PASSWORD));
            user.setPhone(wdUser.getMobile());
            this.save(user);
            // List<WDUserCardInfo> cardList = SsbUtil.getUserCardInfoList(token);
        }

        UserInfo info = new UserInfo();
        info.setSysUserBase(user);
        info.setOpenId(wdUser.getMobile());
        return info;
    }

    @Override
    public boolean saveDto(UserDTO dto) {
        if (NumberUtil.isNotNullOrZero(dto.getUser().getId())) {
            hpUserRoleService.saveBatch(dto.getUser().getId(), dto.getRoleList());
        }

        return true;
    }

    @Override
    public boolean exist(SysUserBase user) {
        if (StrUtil.isBlank(user.getUserName())) {
            return false;
        }
        LambdaQueryWrapper<SysUserBase> wrapper = Wrappers.lambdaQuery();
        wrapper.ne(NumberUtil.isNotNullOrZero(user.getId()), SysUserBase::getId, user.getId())
                .eq(SysUserBase::getUserName, user.getUserName());
        return this.count(wrapper) > 0;
    }

    @Override
    public void initRedisByUserName(String userName) {
        String key = RedisCacheConstant.USER_DETAILS + "::" + userName + ":*";
        stringRedisTemplate.delete(stringRedisTemplate.keys(key));
    }

    private UserInfo getByPhone(String phone) {
        LambdaQueryWrapper<SysUserBase> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(SysUserBase::getPhone, phone);
        SysUserBase user = this.getOne(wrapper);
        // 未注册用户默认注册
        if (user == null) {
            user = new SysUserBase();
            user.setOpenId(phone);
            user.setStatus(Constant.BYTE_NO);
            user.setSource(UserConstant.USER_SOURCE_MOBILE);
            user.setPasswd(ENCODER.encode(Constant.DEFAULT_PASSWORD));
            user.setPhone(phone);
            this.save(user);
        }

        UserInfo info = new UserInfo();
        info.setSysUserBase(user);
        info.setOpenId(phone);
        return info;
    }


    private UserInfo getByOpenId(String openId) {
        LambdaQueryWrapper<SysUserBase> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(SysUserBase::getOpenId, openId);
        SysUserBase user = this.getOne(wrapper);
        // 未注册用户默认注册
        if (user == null) {
            user = new SysUserBase();
            user.setOpenId(openId);
            user.setStatus(Constant.BYTE_NO);
            user.setPasswd(Constant.DEFAULT_PASSWORD);
            this.save(user);
        }

        UserInfo info = new UserInfo();
        info.setSysUserBase(user);
        info.setOpenId(openId);
        return info;
    }
}
